package servlet;
import java.sql.*;

/*
 * Class for managing the database.
 */
public class Database implements AutoCloseable {
	
	// The connection to the database
	private Connection conn = null;
	
	public Database() {
		// Connect to sqlite
		try {
			Class.forName("org.sqlite.JDBC");
			conn = DriverManager.getConnection("jdbc:sqlite:database.db");
			System.out.println("Opened database successfully");
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}
	
	public boolean addName(String name) {
		String sql = "insert into Respondents (name) values(?)";

		try(PreparedStatement ps = conn.prepareStatement(sql)) {
			ps.setString(1, name);
		    ps.executeUpdate(); 
		    return true;
		} catch (SQLException e) {
		    if(e.getErrorCode()==1062 && e.getSQLState().equals("23000")){ 
		    	// duplicate key error
		    	System.out.println(name + " already exists in the database");
		    } else {
				printSqlError(e);
		    }
			return false;
		}
		// Using the try-with-resources syntax, this will call ps.close() automatically
	}
	
	private void printSqlError(SQLException e){
	    System.out.println("SQLException: " + e.getMessage());
	    System.out.println("SQLState: " + e.getSQLState());
	    System.out.println("VendorError: " + e.getErrorCode());
		throw new RuntimeException(e);
	}

	public void close() throws SQLException {
		conn.close();
	}
}
